#!/usr/bin/env python
#
#  A python package for interpreting METAR and SPECI weather reports.
#
#  US conventions for METAR/SPECI reports are described in chapter 12 of
#  the Federal Meteorological Handbook No.1. (FMH-1 1995), issued by NOAA.
#  See <http://metar.noaa.gov/>
#
#  International conventions for the METAR and SPECI codes are specified in
#  the WMO Manual on Codes, vol I.1, Part A (WMO-306 I.i.A).
#
#  This module handles a reports that follow the US conventions, as well
#  the more general encodings in the WMO spec.  Other regional conventions
#  are not supported at present.
#
#  The current METAR report for a given station is available at the URL
#  http://weather.noaa.gov/pub/data/observations/metar/stations/<station>.TXT
#  where <station> is the four-letter ICAO station code.
#
#  The METAR reports for all reporting stations for any "cycle" (i.e., hour)
#  in the last 24 hours is available in a single file at the URL
#  http://weather.noaa.gov/pub/data/observations/metar/cycles/<cycle>Z.TXT
#  where <cycle> is a 2-digit cycle number (e.g., "00", "05" or "23").
#
#  Copyright 2004  Tom Pollard
#
"""
This module defines the Metar class.  A Metar object represents the weather report encoded by a single METAR code.
"""

__author__ = "Tom Pollard"

__email__ = "pollard@alum.mit.edu"

__LICENSE__ = """
Copyright (c) 2004-2016, %s
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
""" % __author__

import re
import datetime
import warnings
import logging

from metar import __version__
from metar.Datatypes import *

## logger
_logger = logging.getLogger(__name__)

## Exceptions

class ParserError(Exception):
  """Exception raised when an unparseable group is found in body of the report."""
  pass

## regular expressions to decode various groups of the METAR code

MISSING_RE = re.compile(r"^[M/]+$")

TYPE_RE =     re.compile(r"^(?P<type>METAR|SPECI)\s+")
STATION_RE =  re.compile(r"^(?P<station>[A-Z][A-Z0-9]{3})\s+")
TIME_RE = re.compile(r"""^(?P<day>\d\d)
                          (?P<hour>\d\d)
                          (?P<min>\d\d)Z?\s+""",
                          re.VERBOSE)
MODIFIER_RE = re.compile(r"^(?P<mod>AUTO|FINO|NIL|TEST|CORR?|RTD|CC[A-G])\s+")
WIND_RE = re.compile(r"""^(?P<dir>[\dO]{3}|[0O]|///|MMM|VRB)
                          (?P<speed>P?[\dO]{2,3}|[/M]{2,3})
                        (G(?P<gust>P?(\d{1,3}|[/M]{1,3})))?
                          (?P<units>KTS?|LT|K|T|KMH|MPS)?
                      (\s+(?P<varfrom>\d\d\d)V
                          (?P<varto>\d\d\d))?\s+""",
                          re.VERBOSE)
VISIBILITY_RE = re.compile(r"""^(?P<vis>(?P<dist>(M|P)?\d\d\d\d|////)
                                        (?P<dir>[NSEW][EW]? | NDV)? |
                                        (?P<distu>(M|P)?(\d+|\d\d?/\d\d?|\d+\s+\d/\d))
                                        (?P<units>SM|KM|M|U) |
                                        CAVOK )\s+""",
                                 re.VERBOSE)
RUNWAY_RE = re.compile(r"""^(RVRNO |
                             R(?P<name>\d\d(RR?|LL?|C)?)/
                              (?P<low>(M|P)?\d\d\d\d)
                            (V(?P<high>(M|P)?\d\d\d\d))?
                              (?P<unit>FT)?[/NDU]*)\s+""",
                              re.VERBOSE)
WEATHER_RE = re.compile(r"""^(?P<int>(-|\+|VC)*)
                             (?P<desc>(MI|PR|BC|DR|BL|SH|TS|FZ)+)?
                             (?P<prec>(DZ|RA|SN|SG|IC|PL|GR|GS|UP|/)*)
                             (?P<obsc>BR|FG|FU|VA|DU|SA|HZ|PY)?
                             (?P<other>PO|SQ|FC|SS|DS|NSW|/+)?
                             (?P<int2>[-+])?\s+""",
                             re.VERBOSE)
SKY_RE= re.compile(r"""^(?P<cover>VV|CLR|SKC|SCK|NSC|NCD|BKN|SCT|FEW|[O0]VC|///)
                        (?P<height>[\dO]{2,4}|///)?
                        (?P<cloud>([A-Z][A-Z]+|///))?\s+""",
                        re.VERBOSE)
TEMP_RE = re.compile(r"""^(?P<temp>(M|-)?\d+|//|XX|MM)/
                          (?P<dewpt>(M|-)?\d+|//|XX|MM)?\s+""",
                          re.VERBOSE)
PRESS_RE = re.compile(r"""^(?P<unit>A|Q|QNH)?
                           (?P<press>[\dO]{3,4}|////)
                           (?P<unit2>INS)?\s+""",
                           re.VERBOSE)
RECENT_RE = re.compile(r"""^RE(?P<desc>MI|PR|BC|DR|BL|SH|TS|FZ)?
                              (?P<prec>(DZ|RA|SN|SG|IC|PL|GR|GS|UP)*)?
                              (?P<obsc>BR|FG|FU|VA|DU|SA|HZ|PY)?
                              (?P<other>PO|SQ|FC|SS|DS)?\s+""",
                              re.VERBOSE)
WINDSHEAR_RE = re.compile(r"^(WS\s+)?(ALL\s+RWY|RWY(?P<name>\d\d(RR?|L?|C)?))\s+")
COLOR_RE = re.compile(r"""^(BLACK)?(BLU|GRN|WHT|RED)\+?
                        (/?(BLACK)?(BLU|GRN|WHT|RED)\+?)*\s*""",
                        re.VERBOSE)
RUNWAYSTATE_RE = re.compile(r"""((?P<name>\d\d) | R(?P<namenew>\d\d)(RR?|LL?|C)?/?)
                                ((?P<special> SNOCLO|CLRD(\d\d|//)) |
                                 (?P<deposit>(\d|/))
                                 (?P<extent>(\d|/))
                                 (?P<depth>(\d\d|//))
                                 (?P<friction>(\d\d|//)))\s+""",
                             re.VERBOSE)
TREND_RE = re.compile(r"^(?P<trend>TEMPO|BECMG|FCST|NOSIG)\s+")

TRENDTIME_RE = re.compile(r"(?P<when>(FM|TL|AT))(?P<hour>\d\d)(?P<min>\d\d)\s+")

REMARK_RE = re.compile(r"^(RMKS?|NOSPECI|NOSIG)\s+")

## regular expressions for remark groups

AUTO_RE = re.compile(r"^AO(?P<type>\d)\s+")
SEALVL_PRESS_RE = re.compile(r"^SLP(?P<press>\d\d\d)\s+")
PEAK_WIND_RE = re.compile(r"""^P[A-Z]\s+WND\s+
                               (?P<dir>\d\d\d)
                               (?P<speed>P?\d\d\d?)/
                               (?P<hour>\d\d)?
                               (?P<min>\d\d)\s+""",
                               re.VERBOSE)
WIND_SHIFT_RE = re.compile(r"""^WSHFT\s+
                                (?P<hour>\d\d)?
                                (?P<min>\d\d)
                                (\s+(?P<front>FROPA))?\s+""",
                                re.VERBOSE)
PRECIP_1HR_RE = re.compile(r"^P(?P<precip>\d\d\d\d)\s+")
PRECIP_24HR_RE = re.compile(r"""^(?P<type>6|7)
                                 (?P<precip>\d\d\d\d)\s+""",
                                 re.VERBOSE)
PRESS_3HR_RE = re.compile(r"""^5(?P<tend>[0-8])
                                (?P<press>\d\d\d)\s+""",
                                re.VERBOSE)
TEMP_1HR_RE = re.compile(r"""^T(?P<tsign>0|1)
                               (?P<temp>\d\d\d)
                               ((?P<dsign>0|1)
                               (?P<dewpt>\d\d\d))?\s+""",
                               re.VERBOSE)
TEMP_6HR_RE = re.compile(r"""^(?P<type>1|2)
                              (?P<sign>0|1)
                              (?P<temp>\d\d\d)\s+""",
                              re.VERBOSE)
TEMP_24HR_RE = re.compile(r"""^4(?P<smaxt>0|1)
                                (?P<maxt>\d\d\d)
                                (?P<smint>0|1)
                                (?P<mint>\d\d\d)\s+""",
                                re.VERBOSE)
UNPARSED_RE = re.compile(r"(?P<group>\S+)\s+")

LIGHTNING_RE = re.compile(r"""^((?P<freq>OCNL|FRQ|CONS)\s+)?
                             LTG(?P<type>(IC|CC|CG|CA)*)
                                ( \s+(?P<loc>( OHD | VC | DSNT\s+ | \s+AND\s+ |
                                 [NSEW][EW]? (-[NSEW][EW]?)* )+) )?\s+""",
                                re.VERBOSE)

TS_LOC_RE = re.compile(r"""TS(\s+(?P<loc>( OHD | VC | DSNT\s+ | \s+AND\s+ |
                                           [NSEW][EW]? (-[NSEW][EW]?)* )+))?
                                          ( \s+MOV\s+(?P<dir>[NSEW][EW]?) )?\s+""",
                           re.VERBOSE)
SNOWDEPTH_RE = re.compile(r"""^4/(?P<snowdepth>\d\d\d)\s+""")

## translation of weather location codes

loc_terms = [ ("OHD", "overhead"),
              ("DSNT", "distant"),
              ("AND", "and"),
              ("VC", "nearby") ]

def xlate_loc( loc ):
  """Substitute English terms for the location codes in the given string."""
  for code, english in loc_terms:
      loc = loc.replace(code,english)
  return loc

## translation of the sky-condition codes into english

SKY_COVER = { "SKC":"clear",
              "CLR":"clear",
              "NSC":"clear",
              "NCD":"clear",
              "FEW":"a few ",
              "SCT":"scattered ",
              "BKN":"broken ",
              "OVC":"overcast",
              "///":"",
              "VV":"indefinite ceiling" }

CLOUD_TYPE = { "TCU":"towering cumulus",
               "CU":"cumulus",
               "CB":"cumulonimbus",
               "SC":"stratocumulus",
               "CBMAM":"cumulonimbus mammatus",
               "ACC":"altocumulus castellanus",
               "SCSL":"standing lenticular stratocumulus",
               "CCSL":"standing lenticular cirrocumulus",
               "ACSL":"standing lenticular altocumulus" }

## translation of the present-weather codes into english

WEATHER_INT = { "-":"light",
                "+":"heavy",
                "-VC":"nearby light",
                "+VC":"nearby heavy",
                "VC":"nearby" }
WEATHER_DESC = { "MI":"shallow",
                 "PR":"partial",
                 "BC":"patches of",
                 "DR":"low drifting",
                 "BL":"blowing",
                 "SH":"showers",
                 "TS":"thunderstorm",
                 "FZ":"freezing" }
WEATHER_PREC = { "DZ":"drizzle",
                 "RA":"rain",
                 "SN":"snow",
                 "SG":"snow grains",
                 "IC":"ice crystals",
                 "PL":"ice pellets",
                 "GR":"hail",
                 "GS":"snow pellets",
                 "UP":"unknown precipitation",
                 "//":"" }
WEATHER_OBSC = { "BR":"mist",
                 "FG":"fog",
                 "FU":"smoke",
                 "VA":"volcanic ash",
                 "DU":"dust",
                 "SA":"sand",
                 "HZ":"haze",
                 "PY":"spray" }
WEATHER_OTHER = { "PO":"sand whirls",
                  "SQ":"squalls",
                  "FC":"funnel cloud",
                  "SS":"sandstorm",
                  "DS":"dust storm" }

WEATHER_SPECIAL = { "+FC":"tornado" }

COLOR = { "BLU":"blue",
          "GRN":"green",
          "WHT":"white" }

## translation of various remark codes into English

PRESSURE_TENDENCY = { "0":"increasing, then decreasing",
                      "1":"increasing more slowly",
                      "2":"increasing",
                      "3":"increasing more quickly",
                      "4":"steady",
                      "5":"decreasing, then increasing",
                      "6":"decreasing more slowly",
                      "7":"decreasing",
                      "8":"decreasing more quickly" }

LIGHTNING_FREQUENCY = { "OCNL":"occasional",
                        "FRQ":"frequent",
                        "CONS":"constant" }
LIGHTNING_TYPE = { "IC":"intracloud",
                   "CC":"cloud-to-cloud",
                   "CG":"cloud-to-ground",
                   "CA":"cloud-to-air" }

REPORT_TYPE = { "METAR":"routine report",
                "SPECI":"special report",
                "AUTO":"automatic report",
                "COR":"manually corrected report" }

## Helper functions

def _report_match(handler, match):
  """Report success or failure of the given handler function. (DEBUG)"""
  if match:
      _logger.debug("%s matched '%s'", handler.__name__, match)
  else:
      _logger.debug("%s didn't match...", handler.__name__)


def _unparsedGroup( self, d ):
    """
    Handle otherwise unparseable main-body groups.
    """
    self._unparsed_groups.append(d['group'])

## METAR report objects

debug = False

class Metar(object):
  """METAR (aviation meteorology report)"""

  def __init__(self, metarcode, month=None, year=None, utcdelta=None,
               strict=True):
      """
      Parse raw METAR code.

      Parameters
      ----------
      metarcode : str
      month, year : int, optional
        Date values to be used when parsing a non-current METAR code. If not
        provided, then the month and year are guessed from the current date.
      utcdelta : int or datetime.timedelta, optional
        An int of hours or a timedelta object used to specify the timezone.
      strict : bool (default is True)
        This option determines if a ``ParserError`` is raised when
        unparsable groups are found or an unexpected exception is encountered.
        Setting this to `False` will prevent exceptions from being raised and
        only generate warning messages.
      """

      self.code = metarcode              # original METAR code
      self.type = 'METAR'                # METAR (routine) or SPECI (special)
      self.mod = "AUTO"                  # AUTO (automatic) or COR (corrected)
      self.station_id = None             # 4-character ICAO station code
      self.time = None                   # observation time [datetime]
      self.cycle = None                  # observation cycle (0-23) [int]
      self.wind_dir = None               # wind direction [direction]
      self.wind_speed = None             # wind speed [speed]
      self.wind_gust = None              # wind gust speed [speed]
      self.wind_dir_from = None          # beginning of range for win dir [direction]
      self.wind_dir_to = None            # end of range for wind dir [direction]
      self.vis = None                    # visibility [distance]
      self.vis_dir = None                # visibility direction [direction]
      self.max_vis = None                # visibility [distance]
      self.max_vis_dir = None            # visibility direction [direction]
      self.temp = None                   # temperature (C) [temperature]
      self.dewpt = None                  # dew point (C) [temperature]
      self.press = None                  # barometric pressure [pressure]
      self.runway = []                   # runway visibility (list of tuples)
      self.weather = []                  # present weather (list of tuples)
      self.recent = []                   # recent weather (list of tuples)
      self.sky = []                      # sky conditions (list of tuples)
      self.windshear = []                # runways w/ wind shear (list of strings)
      self.wind_speed_peak = None        # peak wind speed in last hour
      self.wind_dir_peak = None          # direction of peak wind speed in last hour
      self.peak_wind_time = None         # time of peak wind observation [datetime]
      self.wind_shift_time = None        # time of wind shift [datetime]
      self.max_temp_6hr = None           # max temp in last 6 hours
      self.min_temp_6hr = None           # min temp in last 6 hours
      self.max_temp_24hr = None          # max temp in last 24 hours
      self.min_temp_24hr = None          # min temp in last 24 hours
      self.press_sea_level = None        # sea-level pressure
      self.precip_1hr = None             # precipitation over the last hour
      self.precip_3hr = None             # precipitation over the last 3 hours
      self.precip_6hr = None             # precipitation over the last 6 hours
      self.precip_24hr = None            # precipitation over the last 24 hours
      self.snowdepth = None              # snow depth (distance)
      self._trend = False                # trend groups present (bool)
      self._trend_groups = []            # trend forecast groups
      self._remarks = []                 # remarks (list of strings)
      self._unparsed_groups = []
      self._unparsed_remarks = []

      self._now = datetime.datetime.utcnow()
      if utcdelta:
          self._utcdelta = utcdelta
      else:
          self._utcdelta = datetime.datetime.now() - self._now

      self._month = month
      self._year = year

      code = self.code+" "    # (the regexps all expect trailing spaces...)
      try:
          ngroup = len(self.handlers)
          igroup = 0
          ifailed = -1
          while igroup < ngroup and code:
              pattern, handler, repeatable = self.handlers[igroup]
              if debug:
                  _logger.debug("%s: %s", handler.__name__, code)
              m = pattern.match(code)
              while m:
                  ifailed = -1
                  if debug:
                      _report_match(handler, m.group())
                  handler(self,m.groupdict())
                  code = code[m.end():]
                  if self._trend:
                      code = self._do_trend_handlers(code)
                  if not repeatable:
                      break

                  if debug:
                      _logger.debug("%s: %s", handler.__name__, code)
                  m = pattern.match(code)
              if not m and ifailed < 0:
                  ifailed = igroup
              igroup += 1
              if igroup == ngroup and not m:
                  pattern, handler = (UNPARSED_RE, _unparsedGroup)
                  if debug:
                      _logger.debug("%s: %s", handler.__name__, code)
                  m = pattern.match(code)
                  if debug:
                      _report_match(handler, m.group())
                  handler(self,m.groupdict())
                  code = code[m.end():]
                  igroup = ifailed
                  ifailed = -2  # if it's still -2 when we run out of main-body
                                #  groups, we'll try parsing this group as a remark
          if pattern == REMARK_RE or self.press:
              while code:
                  for pattern, handler in self.remark_handlers:
                      if debug:
                          _logger.debug("%s: %s", handler.__name__, code)
                      m = pattern.match(code)
                      if m:
                          if debug:
                              _report_match(handler, m.group())
                          handler(self,m.groupdict())
                          code = pattern.sub("",code,1)
                          break

      except Exception as err:
          message = (
              ("%s failed while processing '%s'\n\t%s"
               ) % (handler.__name__, code, "\n\t".join(err.args))
          )
          if strict:
              raise ParserError(message)
          else:
              warnings.warn(message, RuntimeWarning)

      if self._unparsed_groups:
          code = ' '.join(self._unparsed_groups)
          message = "Unparsed groups in body '%s' while processing '%s'" % (code, metarcode)
          if strict:
              raise ParserError(message)
          else:
              warnings.warn(message, RuntimeWarning)

  @property
  def decode_completed(self):
      """
      Indicate whether the decoding was complete for non-remark elements.
      """
      return not self._unparsed_groups

  def _do_trend_handlers(self, code):
      for pattern, handler, repeatable in self.trend_handlers:
          if debug: print(handler.__name__,":",code)
          m = pattern.match(code)
          while m:
              if debug: _report_match(handler, m.group())
              self._trend_groups.append(m.group().strip())
              handler(self,m.groupdict())
              code = code[m.end():]
              if not repeatable: break
              m = pattern.match(code)
      return code

  def __str__(self):
      return self.string()

  def _handleType( self, d ):
      """
      Parse the code-type group.

      The following attributes are set:
          type   [string]
      """
      self.type = d['type']

  def _handleStation( self, d ):
      """
      Parse the station id group.

      The following attributes are set:
          station_id   [string]
      """
      self.station_id = d['station']

  def _handleModifier( self, d ):
      """
      Parse the report-modifier group.

      The following attributes are set:
          mod   [string]
      """
      mod = d['mod']
      if mod == 'CORR': mod = 'COR'
      if mod == 'NIL' or mod == 'FINO': mod = 'NO DATA'
      self.mod = mod

  def _handleTime( self, d ):
      """
      Parse the observation-time group.

      The following attributes are set:
          time   [datetime]
          cycle  [int]
          _day   [int]
          _hour  [int]
          _min   [int]
      """
      self._day = int(d['day'])
      if not self._month:
          self._month = self._now.month
          if self._day > self._now.day:
              if self._month == 1:
                  self._month = 12
              else:
                  self._month = self._month - 1
      if not self._year:
          self._year = self._now.year
          if self._month > self._now.month:
              self._year = self._year - 1
          elif self._month == self._now.month and self._day > self._now.day:
              self._year = self._year - 1
      self._hour = int(d['hour'])
      self._min = int(d['min'])
      self.time = datetime.datetime(self._year, self._month, self._day,
                                                                  self._hour, self._min)
      if self._min < 45:
          self.cycle = self._hour
      else:
          self.cycle = self._hour+1

  def _handleWind( self, d ):
      """
      Parse the wind and variable-wind groups.

      The following attributes are set:
          wind_dir           [direction]
          wind_speed         [speed]
          wind_gust          [speed]
          wind_dir_from      [int]
          wind_dir_to        [int]
      """
      wind_dir = d['dir'].replace('O','0')
      if wind_dir != "VRB" and wind_dir != "///" and wind_dir != "MMM":
          self.wind_dir = direction(wind_dir)
      wind_speed = d['speed'].replace('O','0')
      units = d['units']
      if units == 'KTS' or units == 'K' or units == 'T' or units == 'LT':
          units = 'KT'
      if wind_speed.startswith("P"):
          self.wind_speed = speed(wind_speed[1:], units, ">")
      elif not MISSING_RE.match(wind_speed):
          self.wind_speed = speed(wind_speed, units)
      if d['gust']:
          wind_gust = d['gust']
          if wind_gust.startswith("P"):
              self.wind_gust = speed(wind_gust[1:], units, ">")
          elif not MISSING_RE.match(wind_gust):
              self.wind_gust = speed(wind_gust, units)
      if d['varfrom']:
          self.wind_dir_from = direction(d['varfrom'])
          self.wind_dir_to = direction(d['varto'])

  def _handleVisibility( self, d ):
      """
      Parse the minimum and maximum visibility groups.

      The following attributes are set:
          vis          [distance]
          vis_dir      [direction]
          max_vis      [distance]
          max_vis_dir  [direction]
      """
      vis = d['vis']
      vis_less = None
      vis_dir = None
      vis_units = "M"
      vis_dist = "10000"
      if d['dist'] and d['dist'] != '////':
          vis_dist = d['dist']
          if d['dir'] and d['dir'] != 'NDV':
              vis_dir = d['dir']
      elif d['distu']:
          vis_dist = d['distu']
          if d['units'] and d['units'] != "U":
                  vis_units = d['units']
      if vis_dist == "9999":
          vis_dist = "10000"
          vis_less = ">"
      if self.vis:
          if vis_dir:
              self.max_vis_dir = direction(vis_dir)
          self.max_vis = distance(vis_dist, vis_units, vis_less)
      else:
          if vis_dir:
              self.vis_dir = direction(vis_dir)
          self.vis = distance(vis_dist, vis_units, vis_less)

  def _handleRunway(self, d):
      """
      Parse a runway visual range group.

      The following attributes are set:
          range   [list of tuples]
          . name  [string]
          . low   [distance]
          . high  [distance]
          . unit  [string]
      """
      if d['name'] is None:
          return
      unit = d['unit'] if d['unit'] is not None else 'FT'
      low = distance(d['low'], unit)
      if d['high'] is None:
          high = low
      else:
          high = distance(d['high'], unit)
      self.runway.append([d['name'], low, high, unit])

  def _handleWeather( self, d ):
      """
      Parse a present-weather group.

      The following attributes are set:
          weather    [list of tuples]
          .  intensity     [string]
          .  description   [string]
          .  precipitation [string]
          .  obscuration   [string]
          .  other         [string]
      """
      inteni = d['int']
      if not inteni and d['int2']:
          inteni = d['int2']
      desci = d['desc']
      preci = d['prec']
      obsci = d['obsc']
      otheri = d['other']
      self.weather.append((inteni,desci,preci,obsci,otheri))

  def _handleSky( self, d ):
      """
      Parse a sky-conditions group.

      The following attributes are set:
          sky        [list of tuples]
          .  cover   [string]
          .  height  [distance]
          .  cloud   [string]
      """
      height = d['height']
      if not height or height == "///":
          height = None
      else:
          height = height.replace('O','0')
          height = distance(int(height)*100,"FT")
      cover = d['cover']
      if cover == 'SCK' or cover == 'SKC' or cover == 'CL': cover = 'CLR'
      if cover == '0VC': cover = 'OVC'
      cloud = d['cloud']
      if cloud == '///': cloud = ""
      self.sky.append((cover,height,cloud))

  def _handleTemp( self, d ):
      """
      Parse a temperature-dewpoint group.

      The following attributes are set:
          temp    temperature (Celsius) [float]
          dewpt   dew point (Celsius) [float]
      """
      temp = d['temp']
      dewpt = d['dewpt']
      if  temp and temp != "//" and temp != "XX" and temp != "MM" :
          self.temp = temperature(temp)
      if dewpt and dewpt != "//" and dewpt != "XX" and dewpt != "MM" :
          self.dewpt = temperature(dewpt)

  def _handlePressure( self, d ):
      """
      Parse an altimeter-pressure group.

      The following attributes are set:
          press    [int]
      """
      press = d['press']
      if press != '////':
          press = float(press.replace('O','0'))
          if d['unit']:
              if d['unit'] == 'A' or (d['unit2'] and d['unit2'] == 'INS'):
                  self.press = pressure(press/100,'IN')
              elif d['unit'] == 'SLP':
                  if press < 500:
                      press = press/10 + 1000
                  else:
                      press = press/10 + 900
                  self.press = pressure(press,'MB')
                  self._remarks.append("sea-level pressure %.1fhPa" % press)
              else:
                  self.press = pressure(press,'MB')
          elif press > 2500:
              self.press = pressure(press/100,'IN')
          else:
              self.press = pressure(press,'MB')

  def _handleRecent( self, d ):
      """
      Parse a recent-weather group.

      The following attributes are set:
          weather    [list of tuples]
          .  intensity     [string]
          .  description   [string]
          .  precipitation [string]
          .  obscuration   [string]
          .  other         [string]
      """
      desci = d['desc']
      preci = d['prec']
      obsci = d['obsc']
      otheri = d['other']
      self.recent.append(("",desci,preci,obsci,otheri))

  def _handleWindShear( self, d ):
      """
      Parse wind-shear groups.

      The following attributes are set:
          windshear    [list of strings]
      """
      if d['name']:
          self.windshear.append(d['name'])
      else:
          self.windshear.append("ALL")

  def _handleColor( self, d ):
      """
      Parse (and ignore) the color groups.

      The following attributes are set:
          trend    [list of strings]
      """
      pass

  def _handleRunwayState( self, d ):
      """
      Parse (and ignore) the runway state.

      The following attributes are set:
      """
      pass

  def _handleTrend( self, d ):
      """
      Parse (and ignore) the trend groups.
      """
      if 'trend' in d:
          self._trend_groups.append(d['trend'])
      self._trend = True

  def _startRemarks( self, d ):
      """
      Found the start of the remarks section.
      """
      self._remarks = []

  def _handleSealvlPressRemark( self, d ):
      """
      Parse the sea-level pressure remark group.
      """
      value = float(d['press'])/10.0
      if value < 50:
          value += 1000
      else:
          value += 900
      self.press_sea_level = pressure(value,"MB")

  def _handlePrecip24hrRemark( self, d ):
      """
      Parse a 3-, 6- or 24-hour cumulative preciptation remark group.
      """
      value = float(d['precip'])/100.0
      if d['type'] == "6":
          if self.cycle == 3 or self.cycle == 9 or self.cycle == 15 or self.cycle == 21:
              self.precip_3hr = precipitation(value,"IN")
          else:
              self.precip_6hr = precipitation(value,"IN")
      else:
          self.precip_24hr = precipitation(value,"IN")

  def _handlePrecip1hrRemark( self, d ):
      """Parse an hourly precipitation remark group."""
      value = float(d['precip'])/100.0
      self.precip_1hr = precipitation(value,"IN")

  def _handleTemp1hrRemark( self, d ):
      """
      Parse a temperature & dewpoint remark group.

      These values replace the temp and dewpt from the body of the report.
      """
      value = float(d['temp'])/10.0
      if d['tsign'] == "1": value = -value
      self.temp = temperature(value)
      if d['dewpt']:
          value2 = float(d['dewpt'])/10.0
          if d['dsign'] == "1": value2 = -value2
          self.dewpt = temperature(value2)

  def _handleTemp6hrRemark( self, d ):
      """
      Parse a 6-hour maximum or minimum temperature remark group.
      """
      value = float(d['temp'])/10.0
      if d['sign'] == "1": value = -value
      if d['type'] == "1":
          self.max_temp_6hr = temperature(value,"C")
      else:
          self.min_temp_6hr = temperature(value,"C")

  def _handleTemp24hrRemark( self, d ):
      """
      Parse a 24-hour maximum/minimum temperature remark group.
      """
      value = float(d['maxt'])/10.0
      if d['smaxt'] == "1": value = -value
      value2 = float(d['mint'])/10.0
      if d['smint'] == "1": value2 = -value2
      self.max_temp_24hr = temperature(value,"C")
      self.min_temp_24hr = temperature(value2,"C")

  def _handlePress3hrRemark( self, d ):
      """
      Parse a pressure-tendency remark group.
      """
      value = float(d['press'])/10.0
      descrip = PRESSURE_TENDENCY[d['tend']]
      self._remarks.append("3-hr pressure change %.1fhPa, %s" % (value,descrip))

  def _handlePeakWindRemark( self, d ):
      """
      Parse a peak wind remark group.
      """
      peak_dir = int(d['dir'])
      peak_speed = int(d['speed'])
      self.wind_speed_peak = speed(peak_speed, "KT")
      self.wind_dir_peak = direction(peak_dir)
      peak_min = int(d['min'])
      if d['hour']:
          peak_hour = int(d['hour'])
      else:
          peak_hour = self._hour
      self.peak_wind_time = datetime.datetime(self._year, self._month, self._day,
                                                                  peak_hour, peak_min)
      if self.peak_wind_time > self.time:
          if peak_hour > self._hour:
              self.peak_wind_time -= datetime.timedelta(hours=24)
          else:
              self.peak_wind_time -= datetime.timedelta(hours=1)
      self._remarks.append("peak wind %dkt from %d degrees at %d:%02d" % \
                                              (peak_speed, peak_dir, peak_hour, peak_min))

  def _handleWindShiftRemark( self, d ):
      """
      Parse a wind shift remark group.
      """
      if d['hour']:
          wshft_hour = int(d['hour'])
      else:
          wshft_hour = self._hour
      wshft_min = int(d['min'])
      self.wind_shift_time = datetime.datetime(self._year, self._month, self._day,
                                                                  wshft_hour, wshft_min)
      if self.wind_shift_time > self.time:
          if wshft_hour > self._hour:
              self.wind_shift_time -= datetime.timedelta(hours=24)
          else:
              self.wind_shift_time -= datetime.timedelta(hours=1)
      text = "wind shift at %d:%02d" %  (wshft_hour, wshft_min)
      if d['front']:
          text += " (front)"
      self._remarks.append(text)

  def _handleLightningRemark( self, d ):
      """
      Parse a lightning observation remark group.
      """
      parts = []
      if d['freq']:
          parts.append(LIGHTNING_FREQUENCY[d['freq']])
      parts.append("lightning")
      if d['type']:
          ltg_types = []
          group = d['type']
          while group:
              ltg_types.append(LIGHTNING_TYPE[group[:2]])
              group = group[2:]
          parts.append("("+",".join(ltg_types)+")")
      if d['loc']:
          parts.append(xlate_loc(d['loc']))
      self._remarks.append(" ".join(parts))

  def _handleTSLocRemark( self, d ):
      """
      Parse a thunderstorm location remark group.
      """
      text = "thunderstorm"
      if d['loc']:
          text += " "+xlate_loc(d['loc'])
      if d['dir']:
          text += " moving %s" % d['dir']
      self._remarks.append(text)

  def _handleAutoRemark( self, d ):
      """
      Parse an automatic station remark group.
      """
      if d['type'] == "1":
          self._remarks.append("Automated station")
      elif d['type'] == "2":
          self._remarks.append("Automated station (type 2)")

  def _handleSnowDepthRemark(self, d):
      """
      Parse the 4/ group snowdepth report
      """
      self.snowdepth = distance(float(d['snowdepth']), 'IN')
      self._remarks.append(" snowdepth %s" % (self.snowdepth, ))

  def _unparsedRemark( self, d ):
      """
      Handle otherwise unparseable remark groups.
      """
      self._unparsed_remarks.append(d['group'])

  ## the list of handler functions to use (in order) to process a METAR report

  handlers = [ (TYPE_RE, _handleType, False),
               (STATION_RE, _handleStation, False),
               (TIME_RE, _handleTime, False),
               (MODIFIER_RE, _handleModifier, False),
               (WIND_RE, _handleWind, False),
               (VISIBILITY_RE, _handleVisibility, True),
               (RUNWAY_RE, _handleRunway, True),
               (WEATHER_RE, _handleWeather, True),
               (SKY_RE, _handleSky, True),
               (TEMP_RE, _handleTemp, False),
               (PRESS_RE, _handlePressure, True),
               (RECENT_RE,_handleRecent, True),
               (WINDSHEAR_RE, _handleWindShear, True),
               (COLOR_RE, _handleColor, True),
               (RUNWAYSTATE_RE, _handleRunwayState, True),
               (TREND_RE, _handleTrend, False),
               (REMARK_RE, _startRemarks, False) ]

  trend_handlers = [ (TRENDTIME_RE, _handleTrend, True),
                     (WIND_RE, _handleTrend, True),
                     (VISIBILITY_RE, _handleTrend, True),
                     (WEATHER_RE, _handleTrend, True),
                     (SKY_RE, _handleTrend, True),
                     (COLOR_RE, _handleTrend, True)]

  ## the list of patterns for the various remark groups,
  ## paired with the handler functions to use to record the decoded remark.

  remark_handlers = [ (AUTO_RE,         _handleAutoRemark),
                      (SEALVL_PRESS_RE, _handleSealvlPressRemark),
                      (PEAK_WIND_RE,    _handlePeakWindRemark),
                      (WIND_SHIFT_RE,   _handleWindShiftRemark),
                      (LIGHTNING_RE,    _handleLightningRemark),
                      (TS_LOC_RE,       _handleTSLocRemark),
                      (TEMP_1HR_RE,     _handleTemp1hrRemark),
                      (PRECIP_1HR_RE,   _handlePrecip1hrRemark),
                      (PRECIP_24HR_RE,  _handlePrecip24hrRemark),
                      (PRESS_3HR_RE,    _handlePress3hrRemark),
                      (TEMP_6HR_RE,     _handleTemp6hrRemark),
                      (TEMP_24HR_RE,    _handleTemp24hrRemark),
                      (SNOWDEPTH_RE,    _handleSnowDepthRemark),
                      (UNPARSED_RE,     _unparsedRemark) ]

  ## functions that return text representations of conditions for output

  def string( self ):
      """
      Return a human-readable version of the decoded report.
      """
      lines = []
      lines.append("station: %s" % self.station_id)
      if self.type:
          lines.append("type: %s" % self.report_type())
      if self.time:
          lines.append("time: %s" % self.time.ctime())
      if self.temp:
          lines.append("temperature: %s" % self.temp.string("C"))
      if self.dewpt:
          lines.append("dew point: %s" % self.dewpt.string("C"))
      if self.wind_speed:
          lines.append("wind: %s" % self.wind())
      if self.wind_speed_peak:
          lines.append("peak wind: %s" % self.peak_wind())
      if self.wind_shift_time:
          lines.append("wind shift: %s" % self.wind_shift())
      if self.vis:
          lines.append("visibility: %s" % self.visibility())
      if self.runway:
          lines.append("visual range: %s" % self.runway_visual_range())
      if self.press:
          lines.append("pressure: %s" % self.press.string("mb"))
      if self.weather:
          lines.append("weather: %s" % self.present_weather())
      if self.sky:
          lines.append("sky: %s" % self.sky_conditions("\n     "))
      if self.press_sea_level:
          lines.append("sea-level pressure: %s" % self.press_sea_level.string("mb"))
      if self.max_temp_6hr:
          lines.append("6-hour max temp: %s" % str(self.max_temp_6hr))
      if self.max_temp_6hr:
          lines.append("6-hour min temp: %s" % str(self.min_temp_6hr))
      if self.max_temp_24hr:
          lines.append("24-hour max temp: %s" % str(self.max_temp_24hr))
      if self.max_temp_24hr:
          lines.append("24-hour min temp: %s" % str(self.min_temp_24hr))
      if self.precip_1hr:
          lines.append("1-hour precipitation: %s" % str(self.precip_1hr))
      if self.precip_3hr:
          lines.append("3-hour precipitation: %s" % str(self.precip_3hr))
      if self.precip_6hr:
          lines.append("6-hour precipitation: %s" % str(self.precip_6hr))
      if self.precip_24hr:
          lines.append("24-hour precipitation: %s" % str(self.precip_24hr))
      if self._remarks:
          lines.append("remarks:")
          lines.append("- "+self.remarks("\n- "))
      if self._unparsed_remarks:
          lines.append("- "+' '.join(self._unparsed_remarks))
      lines.append("METAR: "+self.code)
      return "\n".join(lines)

  def report_type( self ):
      """
      Return a textual description of the report type.
      """
      if self.type == None:
          text = "unknown report type"
      elif self.type in REPORT_TYPE:
          text  = REPORT_TYPE[self.type]
      else:
          text = self.type+" report"
      if self.cycle:
          text += ", cycle %d" % self.cycle
      if self.mod:
          if self.mod in REPORT_TYPE:
              text += " (%s)" % REPORT_TYPE[self.mod]
          else:
              text += " (%s)" % self.mod
      return text

  def wind( self, units="KT" ):
      """
      Return a textual description of the wind conditions.

      Units may be specified as "MPS", "KT", "KMH", or "MPH".
      """
      if self.wind_speed == None:
          return "missing"
      elif self.wind_speed.value() == 0.0:
          text = "calm"
      else:
          wind_speed = self.wind_speed.string(units)
          if not self.wind_dir:
              text = "variable at %s" % wind_speed
          elif self.wind_dir_from:
              text = "%s to %s at %s" % \
                             (self.wind_dir_from.compass(), self.wind_dir_to.compass(), wind_speed)
          else:
              text = "%s at %s" % (self.wind_dir.compass(), wind_speed)
          if self.wind_gust:
              text += ", gusting to %s" % self.wind_gust.string(units)
      return text

  def peak_wind( self, units="KT" ):
      """
      Return a textual description of the peak wind conditions.

      Units may be specified as "MPS", "KT", "KMH", or "MPH".
      """
      if self.wind_speed_peak == None:
          return "missing"
      elif self.wind_speed_peak.value() == 0.0:
          text = "calm"
      else:
          wind_speed = self.wind_speed_peak.string(units)
          if not self.wind_dir_peak:
              text = wind_speed
          else:
              text = "%s at %s" % (self.wind_dir_peak.compass(), wind_speed)
              if not self.peak_wind_time == None:
                  text += " at %s" % self.peak_wind_time.strftime('%H:%M')
      return text

  def wind_shift( self, units="KT" ):
      """
      Return a textual description of the wind shift time

      Units may be specified as "MPS", "KT", "KMH", or "MPH".
      """
      if self.wind_shift_time == None:
          return "missing"
      else:
          return self.wind_shift_time.strftime('%H:%M')

  def visibility( self, units=None ):
      """
      Return a textual description of the visibility.

      Units may be statute miles ("SM") or meters ("M").
      """
      if self.vis == None:
          return "missing"
      if self.vis_dir:
          text = "%s to %s" % (self.vis.string(units), self.vis_dir.compass())
      else:
          text = self.vis.string(units)
      if self.max_vis:
          if self.max_vis_dir:
              text += "; %s to %s" % (self.max_vis.string(units), self.max_vis_dir.compass())
          else:
              text += "; %s" % self.max_vis.string(units)
      return text

  def runway_visual_range(self, units=None):
      """
      Return a textual description of the runway visual range.
      """
      lines = []
      for name, low, high, unit in self.runway:
          reportunits = unit if units is None else units
          if low != high:
              lines.append(
                  ("on runway %s, from %d to %s"
                   ) % (name, low.value(reportunits),
                        high.string(reportunits))
              )
          else:
              lines.append(
                  "on runway %s, %s" % (name, low.string(reportunits))
              )
      return "; ".join(lines)

  def present_weather( self ):
      """
      Return a textual description of the present weather.
      """
      return self._weather( self.weather )

  def recent_weather( self ):
      """
      Return a textual description of the recent weather.
      """
      return self._weather( self.recent )

  def _weather( self, weather ):
      """
      Return a textual description of weather.
      """
      text_list = []
      for weatheri in weather:
          (inteni,desci,preci,obsci,otheri) = weatheri
          text_parts = []
          code_parts = []
          if inteni:
              code_parts.append(inteni)
              text_parts.append(WEATHER_INT[inteni])
          if desci:
              code_parts.append(desci)
              if desci != "SH" or not preci:
                  text_parts.append(WEATHER_DESC[desci[0:2]])
                  if len(desci) == 4:
                      text_parts.append(WEATHER_DESC[desci[2:]])
          if preci:
              code_parts.append(preci)
              if len(preci) == 2:
                  precip_text = WEATHER_PREC[preci]
              elif len(preci) == 4:
                  precip_text = WEATHER_PREC[preci[:2]]+" and "
                  precip_text += WEATHER_PREC[preci[2:]]
              elif len(preci) == 6:
                  precip_text = WEATHER_PREC[preci[:2]]+", "
                  precip_text += WEATHER_PREC[preci[2:4]]+" and "
                  precip_text += WEATHER_PREC[preci[4:]]
              if desci == "TS":
                  text_parts.append("with")
              text_parts.append(precip_text)
              if desci == "SH":
                  text_parts.append(WEATHER_DESC[desci])
          if obsci:
              code_parts.append(obsci)
              text_parts.append(WEATHER_OBSC[obsci])

          if otheri:
              code_parts.append(otheri)
              text_parts.append(WEATHER_OTHER[otheri])
          code = " ".join(code_parts)
          if code in WEATHER_SPECIAL:
              text_list.append(WEATHER_SPECIAL[code])
          else:
              text_list.append(" ".join(text_parts))
      return "; ".join(text_list)

  def sky_conditions( self, sep="; " ):
      """
      Return a textual description of the sky conditions.
      """
      text_list = []
      for skyi in self.sky:
          (cover,height,cloud) = skyi
          if cover in ["SKC", "CLR", "NSC"]:
              text_list.append(SKY_COVER[cover])
          else:
              if cloud:
                  what = CLOUD_TYPE[cloud]
              elif SKY_COVER[cover].endswith(" "):
                  what = "clouds"
              else:
                  what = ""
              label = "%s %s" % (SKY_COVER[cover], what)
              # HACK here to account for 'empty' entries with above format
              label = " ".join(label.strip().split())
              if cover == "VV":
                  label += ", vertical visibility to %s" % (str(height), )
              else:
                  label += " at %s" % (str(height), )
              text_list.append(label)
      return sep.join(text_list)

  def trend( self ):
      """
      Return the trend forecast groups
      """
      return " ".join(self._trend_groups)

  def remarks( self, sep="; "):
      """
      Return the decoded remarks.
      """
      return sep.join(self._remarks)




#
# Run all metar tests
#
import unittest

def suite():
    modules = ( "test_metar", "test_pressure", "test_speed", 
                "test_temperature", "test_direction", "test_distance",
                "test_precipitation")
    alltests = unittest.TestSuite()
    for module in map(__import__, modules):
        alltests.addTest(unittest.findTestCases(module))
    return alltests

if __name__ == '__main__':
    unittest.main(defaultTest='suite')




#
#  Python classes to represent dimensioned quantities used in weather reports
#
#  Copyright 2004  Tom Pollard
# 
import re
from math import sin, cos, atan2, sqrt

## exceptions

class UnitsError(Exception):
  """Exception raised when unrecognized units are used."""
  pass

## regexp to match fractions (used by distance class)
## [Note: numerator of fraction must be single digit.]

FRACTION_RE = re.compile(r"^((?P<int>\d+)\s*)?(?P<num>\d)/(?P<den>\d+)$")
  
## classes representing dimensioned values in METAR reports
    
class temperature(object):
  """A class representing a temperature value."""
  legal_units = [ "F", "C", "K" ]
  
  def __init__( self, value, units="C" ):
    if not units.upper() in temperature.legal_units:
      raise UnitsError("unrecognized temperature unit: '"+units+"'")
    self._units = units.upper()
    try:
      self._value = float(value)
    except ValueError:
      if value.startswith('M'):
        self._value = -float(value[1:])
      else:
        raise ValueError("temperature must be integer: '"+str(value)+"'")

  def __str__(self):
    return self.string()
    
  def value( self, units=None ):
    """Return the temperature in the specified units."""
    if units == None:
      return self._value
    else:
      if not units.upper() in temperature.legal_units:
        raise UnitsError("unrecognized temperature unit: '"+units+"'")
      units = units.upper()
    if self._units == "C":
      celsius_value = self._value
    elif self._units == "F":
      celsius_value = (self._value-32.0)/1.8
    elif self._units == "K":
      celsius_value = self._value-273.15
    if units == "C":
        return celsius_value
    elif units == "K":
        return 273.15+celsius_value
    elif units == "F":
      return 32.0+celsius_value*1.8
      
  def string( self, units=None ):
    """Return a string representation of the temperature, using the given units."""
    if units == None:
      units = self._units
    else:
      if not units.upper() in temperature.legal_units:
        raise UnitsError("unrecognized temperature unit: '"+units+"'")
      units = units.upper()
    val = self.value(units)
    if units == "C":
      return "%.1f C" % val
    elif units == "F":
      return "%.1f F" % val
    elif units == "K":
      return "%.1f K" % val

class pressure(object):
  """A class representing a barometric pressure value."""
  legal_units = [ "MB", "HPA", "IN" ]
  
  def __init__( self, value, units="MB" ):
    if not units.upper() in pressure.legal_units:
      raise UnitsError("unrecognized pressure unit: '"+units+"'")
    self._value = float(value)
    self._units = units.upper()

  def __str__(self):
    return self.string()
    
  def value( self, units=None ):
    """Return the pressure in the specified units."""
    if units == None:
      return self._value
    else:
      if not units.upper() in pressure.legal_units:
        raise UnitsError("unrecognized pressure unit: '"+units+"'")
      units = units.upper()
    if units == self._units:
      return self._value
    if self._units == "IN":
      mb_value = self._value*33.86398
    else:
      mb_value = self._value
    if units == "MB" or units == "HPA":
      return mb_value
    elif units == "IN":
        return mb_value/33.86398
    else:
      raise UnitsError("unrecognized pressure unit: '"+units+"'")
      
  def string( self, units=None ):
    """Return a string representation of the pressure, using the given units."""
    if not units:
      units = self._units
    else:
      if not units.upper() in pressure.legal_units:
        raise UnitsError("unrecognized pressure unit: '"+units+"'")
      units = units.upper()
    val = self.value(units)
    if units == "MB":
      return "%.1f mb" % val
    elif units == "HPA":
      return "%.1f hPa" % val
    elif units == "IN":
      return "%.2f inches" % val

class speed(object):
  """A class representing a wind speed value."""
  legal_units = [ "KT", "MPS", "KMH", "MPH" ]
  legal_gtlt = [ ">", "<" ]
  
  def __init__( self, value, units=None, gtlt=None ):
    if not units:
      self._units = "MPS"
    else:
      if not units.upper() in speed.legal_units:
        raise UnitsError("unrecognized speed unit: '"+units+"'")
      self._units = units.upper()
    if gtlt and not gtlt in speed.legal_gtlt:
      raise ValueError("unrecognized greater-than/less-than symbol: '"+gtlt+"'")
    self._gtlt = gtlt
    self._value = float(value)

  def __str__(self):
    return self.string()
    
  def value( self, units=None ):
    """Return the pressure in the specified units."""
    if not units:
      return self._value
    else:
      if not units.upper() in speed.legal_units:
        raise UnitsError("unrecognized speed unit: '"+units+"'")
      units = units.upper()
    if units == self._units:
      return self._value
    if self._units == "KMH":
      mps_value = self._value/3.6
    elif self._units == "KT":
      mps_value = self._value*0.514444
    elif self._units == "MPH":
      mps_value = self._value*0.447000
    else:
      mps_value = self._value
    if units == "KMH":
      return mps_value*3.6
    elif units == "KT":
      return mps_value/0.514444
    elif units == "MPH":
      return mps_value/0.447000
    elif units == "MPS":
      return mps_value
      
  def string( self, units=None ):
    """Return a string representation of the speed in the given units."""
    if not units:
      units = self._units
    else:
      if not units.upper() in speed.legal_units:
        raise UnitsError("unrecognized speed unit: '"+units+"'")
      units = units.upper()
    val = self.value(units)
    if units == "KMH":
      text = "%.0f km/h" % val
    elif units == "KT":
      text = "%.0f knots" % val
    elif units == "MPH":
      text = "%.0f mph" % val
    elif units == "MPS":
      text = "%.0f mps" % val
    if self._gtlt == ">":
      text = "greater than "+text
    elif self._gtlt == "<":
      text = "less than "+text
    return text


class distance(object):
  """A class representing a distance value."""
  legal_units = [ "SM", "MI", "M", "KM", "FT", "IN" ]
  legal_gtlt = [ ">", "<" ]
  
  def __init__( self, value, units=None, gtlt=None ):
    if not units:
      self._units = "M"
    else:
      if not units.upper() in distance.legal_units:
        raise UnitsError("unrecognized distance unit: '"+units+"'")
      self._units = units.upper()
    
    try:
      if value.startswith('M'):
        value = value[1:]
        gtlt = "<"
      elif value.startswith('P'):
        value = value[1:]
        gtlt = ">"
    except:
      pass
    if gtlt and not gtlt in distance.legal_gtlt:
      raise ValueError("unrecognized greater-than/less-than symbol: '"+gtlt+"'")
    self._gtlt = gtlt
    try:
      self._value = float(value)
      self._num = None
      self._den = None
    except ValueError:
      mf = FRACTION_RE.match(value)
      if not mf:
        raise ValueError("distance is not parseable: '"+str(value)+"'")
      df = mf.groupdict()
      self._num = int(df['num'])
      self._den = int(df['den'])
      self._value = float(self._num)/float(self._den)
      if df['int']:
        self._value += float(df['int'])

  def __str__(self):
    return self.string()
    
  def value( self, units=None ):
    """Return the distance in the specified units."""
    if not units:
      return self._value
    else:
      if not units.upper() in distance.legal_units:
        raise UnitsError("unrecognized distance unit: '"+units+"'")
      units = units.upper()
    if units == self._units:
      return self._value
    if self._units == "SM" or self._units == "MI":
      m_value = self._value*1609.344
    elif self._units == "FT":
      m_value = self._value/3.28084
    elif self._units == "IN":
      m_value = self._value/39.3701
    elif self._units == "KM":
      m_value = self._value*1000
    else:
      m_value = self._value
    if units == "SM" or units == "MI":
      return m_value/1609.344
    elif units == "FT":
      return m_value*3.28084
    elif units == "IN":
      return m_value*39.3701
    elif units == "KM":
      return m_value/1000
    elif units == "M":
      return m_value
      
  def string( self, units=None ):
    """Return a string representation of the distance in the given units."""
    if not units:
      units = self._units
    else:
      if not units.upper() in distance.legal_units:
        raise UnitsError("unrecognized distance unit: '"+units+"'")
      units = units.upper()
    if self._num and self._den and units == self._units:
      val = int(self._value - self._num/self._den)
      if val:
        text = "%d %d/%d" % (val, self._num, self._den)
      else:
        text = "%d/%d" % (self._num, self._den)
    else:
      if units == "KM":
        text = "%.1f" % self.value(units)
      else:
        text = "%.0f" % self.value(units)
    if units == "SM" or units == "MI":
      text += " miles"
    elif units == "M":
      text += " meters"
    elif units == "KM":
      text += " km"
    elif units == "FT":
      text += " feet"
    elif units == "IN":
      text += " inches"
    if self._gtlt == ">":
      text = "greater than "+text
    elif self._gtlt == "<":
      text = "less than "+text
    return text


class direction(object):
  """A class representing a compass direction."""
  
  compass_dirs = { "N":  0.0, "NNE": 22.5, "NE": 45.0, "ENE": 67.5, 
                   "E": 90.0, "ESE":112.5, "SE":135.0, "SSE":157.5,
                   "S":180.0, "SSW":202.5, "SW":225.0, "WSW":247.5,
                   "W":270.0, "WNW":292.5, "NW":315.0, "NNW":337.5 }

  def __init__( self, d ):
    if d in direction.compass_dirs:
      self._compass = d
      self._degrees = direction.compass_dirs[d]
    else:
      self._compass = None
      value = float(d)
      if value < 0.0 or value > 360.0:
        raise ValueError("direction must be 0..360: '"+str(value)+"'")
      self._degrees = value

  def __str__(self):
    return self.string()
      
  def value( self ):
    """Return the numerical direction, in degrees."""
    return self._degrees
    
  def string( self ):
    """Return a string representation of the numerical direction."""
    return "%.0f degrees" % self._degrees
    
  def compass( self ):
    """Return the compass direction, e.g., "N", "ESE", etc.)."""
    if not self._compass:
      degrees = 22.5 * round(self._degrees/22.5)
      if degrees == 360.0:
        self._compass = "N"
      else:
        for name, d in direction.compass_dirs.items():
          if d == degrees:
            self._compass = name
            break
    return self._compass


class precipitation(object):
  """A class representing a precipitation value."""
  legal_units = [ "IN", "CM" ]
  legal_gtlt = [ ">", "<" ]
  
  def __init__( self, value, units=None, gtlt=None ):
    if not units:
      self._units = "IN"
    else:
      if not units.upper() in precipitation.legal_units:
        raise UnitsError("unrecognized precipitation unit: '"+units+"'")
      self._units = units.upper()
    
    try:
      if value.startswith('M'):
        value = value[1:]
        gtlt = "<"
      elif value.startswith('P'):
        value = value[1:]
        gtlt = ">"
    except:
      pass
    if gtlt and not gtlt in precipitation.legal_gtlt:
      raise ValueError("unrecognized greater-than/less-than symbol: '"+gtlt+"'")
    self._gtlt = gtlt
    self._value = float(value)
    # In METAR world, a string of four zeros denotes trace
    self._istrace = (value == "0000")

  def __str__(self):
    return self.string()
    
  def value( self, units=None ):
    """Return the precipitation in the specified units."""
    if not units:
      return self._value
    else:
      if not units.upper() in precipitation.legal_units:
        raise UnitsError("unrecognized precipitation unit: '"+units+"'")
      units = units.upper()
    if units == self._units:
      return self._value
    if self._units == "CM":
      i_value = self._value*2.54
    else:
      i_value = self._value
    if units == "CM":
      return i_value*2.54
    else:
      return i_value
      
  def string( self, units=None ):
    """Return a string representation of the precipitation in the given units."""
    if not units:
      units = self._units
    else:
      if not units.upper() in precipitation.legal_units:
        raise UnitsError("unrecognized precipitation unit: '"+units+"'")
      units = units.upper()
    # A trace is a trace in any units
    if self._istrace:
      return "Trace"
    text = "%.2f" % self.value(units)
    if units == "CM":
      text += "cm"
    else:
      text += "in"
    if self._gtlt == ">":
      text = "greater than "+text
    elif self._gtlt == "<":
      text = "less than "+text
    return text

  def istrace(self):
    """Return a boolean on if this precipitation was a trace"""
    return self._istrace


class position(object):
  """A class representing a location on the earth's surface."""
   
  def __init__( self, latitude=None, longitude=None ):
    self.latitude = latitude
    self.longitude = longitude

  def __str__(self):
    return self.string()
   
  def getdistance( self, position2 ):
    """
    Calculate the great-circle distance to another location using the Haversine
    formula.  See <http://www.movable-type.co.uk/scripts/LatLong.html>
    and <http://mathforum.org/library/drmath/sets/select/dm_lat_long.html>
    """
    earth_radius = 637100.0
    lat1 = self.latitude
    long1 = self.longitude
    lat2 = position2.latitude
    long2 = position2.longitude
    a = sin(0.5(lat2-lat1)) + cos(lat1)*cos(lat2)*sin(0.5*(long2-long1)**2)
    c = 2.0*atan(sqrt(a)*sqrt(1.0-a))
    d = distance(earth_radius*c,"M")
    return d

  def getdirection( self, position2 ):
    """
    Calculate the initial direction to another location.  (The direction
    typically changes as you trace the great circle path to that location.)
    See <http://www.movable-type.co.uk/scripts/LatLong.html>.
    """
    lat1 = self.latitude
    long1 = self.longitude
    lat2 = position2.latitude
    long2 = position2.longitude
    s = -sin(long1-long2)*cos(lat2)
    c = cos(lat1)*sin(lat2) - sin(lat1)*cos(lat2)*cos(long1-long2)
    d = atan2(s,c)*180.0/math.pi
    if d < 0.0: d += 360.0
    return direction(d)




import unittest
from metar.Datatypes import temperature, UnitsError

class TemperatureTest(unittest.TestCase):
  
  def testDefaults(self):
    self.assertEqual( temperature("32").value(), 32.0 )
    self.assertEqual( temperature("32").value("C"), 32.0 )
    self.assertEqual( temperature("32").string(), "32.0 C" )
    self.assertEqual( temperature("32","F").string(), "32.0 F" )
  
  def testInputs(self):
    self.assertEqual( temperature("32").value(), 32.0 )
    self.assertEqual( temperature(32).value(), 32.0 )
    self.assertEqual( temperature(32.0).value(), 32.0 )
    
    self.assertEqual( temperature("32", "c").value(), 32.0 )
    self.assertEqual( temperature("32", "f").value(), 32.0 )
    self.assertEqual( temperature("32", "k").value(), 32.0 )
    
    self.assertEqual( temperature("50", "F").value("c"), 10.0 )
    self.assertEqual( temperature("50", "f").value("C"), 10.0 )
  
  def testErrorChecking(self):
    self.assertRaises( ValueError, temperature, "32C" )
    self.assertRaises( ValueError, temperature, "M10F" )
    self.assertRaises( UnitsError, temperature, "32", "J" )
    self.assertRaises( UnitsError, temperature("32").value, "J" )
    self.assertRaises( UnitsError, temperature("32").string, "J" )
  
  def testConversions(self):
    self.assertEqual( temperature("32","F").value("F"), 32.0 )
    self.assertEqual( temperature("32","F").value("C"), 0.0 )
    self.assertEqual( temperature("50","F").value("C"), 10.0 )
    self.assertEqual( temperature("32","F").value("K"), 273.15 )
    
    self.assertEqual( temperature("20","C").value("C"), 20.0 )
    self.assertEqual( temperature("M10","C").value("F"), 14.0 )
    self.assertEqual( temperature("M0","C").value("F"), 32.0 )
    self.assertEqual( temperature("20","C").value("K"), 293.15 )
    self.assertEqual( temperature("20","C").value("F"), 68.0 )
    self.assertEqual( temperature("30","C").value("F"), 86.0 )

    self.assertEqual( temperature("263.15","K").value("K"), 263.15 )
    self.assertEqual( temperature("263.15","K").value("C"), -10.0 )
    self.assertEqual( temperature("263.15","K").value("F"), 14.0 )
    
    self.assertEqual( temperature("10", "C").string("C"), "10.0 C" )
    self.assertEqual( temperature("10", "C").string("F"), "50.0 F" )
    self.assertEqual( temperature("10", "C").string("K"), "283.1 K" )
    
if __name__=='__main__':
  unittest.main()
  



import unittest
from metar.Datatypes import direction

def suite():
  return unittest.makeSuite(DirectionTest)

class DirectionTest(unittest.TestCase):
  
  def testUsage(self):
    self.assertEqual( direction("90").value(), 90.0 )
    self.assertEqual( direction(90).value(), 90.0 )
    self.assertEqual( direction(90.0).value(), 90.0 )
    self.assertEqual( direction("90").string(), "90 degrees" )
    self.assertEqual( direction("E").compass(), "E" )
   
  def testErrorChecking(self):
    self.assertRaises( ValueError, direction, "North" )
    self.assertRaises( ValueError, direction, -10 )
    self.assertRaises( ValueError, direction, "361" )
  
  def testConversion(self):
    self.assertEqual( direction("N").value(),     0.0 )
    self.assertEqual( direction("NNE").value(),  22.5 )
    self.assertEqual( direction("NE").value(),   45.0 )
    self.assertEqual( direction("ENE").value(),  67.5 )
    self.assertEqual( direction("E").value(),    90.0 )
    self.assertEqual( direction("ESE").value(), 112.5 )
    self.assertEqual( direction("SE").value(),  135.0 )
    self.assertEqual( direction("SSE").value(), 157.5 )
    self.assertEqual( direction("S").value(),   180.0 )
    self.assertEqual( direction("SSW").value(), 202.5 )
    self.assertEqual( direction("SW").value(),  225.0 )
    self.assertEqual( direction("WSW").value(), 247.5 )
    self.assertEqual( direction("W").value(),   270.0 )
    self.assertEqual( direction("WNW").value(), 292.5 )
    self.assertEqual( direction("NW").value(),  315.0 )
    self.assertEqual( direction("NNW").value(), 337.5 )
    
    self.assertEqual( direction("0").compass(), "N" )
    self.assertEqual( direction("5").compass(), "N" )
    self.assertEqual( direction("355").compass(), "N" )
    self.assertEqual( direction("20").compass(), "NNE" )
    self.assertEqual( direction("60").compass(), "ENE" )
    self.assertEqual( direction("247.5").compass(), "WSW" )
      
if __name__=='__main__':
  unittest.main()
  



import unittest
from metar.Datatypes import precipitation

class PrecipitationTest(unittest.TestCase):

  def test_trace(self):
    """How do we handle trace reports"""
    self.assertEqual( precipitation("0000", "IN").string(), "Trace" )
    self.assertTrue( precipitation("0000", "IN").istrace() )
    self.assertFalse( precipitation("0010", "IN").istrace() )
    
if __name__=='__main__':
  unittest.main()
  



#!/usr/bin/python
#
#  A python package for interpreting METAR and SPECI weather reports.
#  
#  US conventions for METAR/SPECI reports are described in chapter 12 of
#  the Federal Meteorological Handbook No.1. (FMH-1 1995), issued by NOAA. 
#  See <http://metar.noaa.gov/>
# 
#  International conventions for the METAR and SPECI codes are specified in 
#  the WMO Manual on Codes, vol I.1, Part A (WMO-306 I.i.A).  
#
#  This module handles a reports that follow the US conventions, as well
#  the more general encodings in the WMO spec.  Other regional conventions
#  are not supported at present.
#
#  The current METAR report for a given station is available at the URL
#  http://weather.noaa.gov/pub/data/observations/metar/stations/<station>.TXT
#  where <station> is the four-letter ICAO station code.  
#
#  The METAR reports for all reporting stations for any "cycle" (i.e., hour) 
#  in the last 24 hours is available in a single file at the URL
#  http://weather.noaa.gov/pub/data/observations/metar/cycles/<cycle>Z.TXT
#  where <cycle> is a 2-digit cycle number (e.g., "00", "05" or "23").  
# 
#  Copyright 2004-2009  Tom Pollard
#  All rights reserved.
# 
__author__ = "Tom Pollard"

__email__ = "pollard@alum.mit.edu"

__version__ = "1.6.0"

__doc__ = """metar v%s (c) 2009, %s

Metar is a python package that interprets coded METAR and SPECI weather reports.

Please e-mail bug reports to: %s""" % (__version__, __author__,__email__)

__LICENSE__ = """
Copyright (c) 2009, %s
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
""" % __author__




#!/usr/bin/python
#
#  Python module to provide station information from the ICAO identifiers
#
#  Copyright 2004  Tom Pollard
# 
from math import sin, cos, atan2, sqrt
from datatypes import position, distance, direction

class station:
  """An object representing a weather station."""
  
  def __init__( self, id, city=None, state=None, country=None, latitude=None, longitude=None):
    self.id = id
    self.city = city
    self.state = state
    self.country = country
    self.position = position(latitude,longitude)
    if self.state:
      self.name = "%s, %s" % (self.city, self.state)
    else:
      self.name = self.city
    
station_file_name = "nsd_cccc.txt"
station_file_url = "http://www.noaa.gov/nsd_cccc.txt"

stations = {}

fh = open(station_file_name,'r')
for line in fh:
  f = line.strip().split(";")
  stations[f[0]] = station(f[0],f[3],f[4],f[5],f[7],f[8])
fh.close()

if __name__ == "__main__":
  for id in [ 'KEWR', 'KIAD', 'KIWI', 'EKRK' ]:
    print(id, stations[id].name, stations[id].country)




import unittest
from metar.Datatypes import pressure, UnitsError

class PressureTest(unittest.TestCase):
  
  def testDefaults(self):
    self.assertEqual( pressure("1000").value(), 1000.0 )
    self.assertEqual( pressure("1000","HPA").value(), 1000.0 )
    self.assertEqual( pressure("30","in").value(), 30.0 )
    self.assertEqual( pressure("30","in").string(), "30.00 inches" )
    self.assertEqual( pressure("1000").value("MB"), 1000 )
    self.assertEqual( pressure("1000").string(), "1000.0 mb" )
    self.assertEqual( pressure("1000","HPA").string(), "1000.0 hPa" )
  
  def testInputs(self):
    self.assertEqual( pressure("1000").value(), 1000.0 )
    self.assertEqual( pressure("1000.0").value(), 1000.0 )
    self.assertEqual( pressure(1000).value(), 1000.0 )
    self.assertEqual( pressure(1000.0).value(), 1000.0 )
    
    self.assertEqual( pressure("1000", "mb").value(), 1000.0 )
    self.assertEqual( pressure("1000", "hPa").value(), 1000.0 )
    self.assertEqual( pressure("30.00", "in").value(), 30.0 )
    
    self.assertEqual( pressure("1000", "MB").value("MB"), 1000.0 )
    self.assertEqual( pressure("1000", "MB").value("HPA"), 1000.0 )
    self.assertEqual( pressure("1000", "HPA").value("mb"), 1000.0 )
  
  def testErrorChecking(self):
    self.assertRaises( ValueError, pressure, "A2995" )
    self.assertRaises( UnitsError, pressure, "1000", "bars" )
    self.assertRaises( UnitsError, pressure("30.00").value, "psi" )
    self.assertRaises( UnitsError, pressure("32.00").string, "atm" )
  
  def testConversions(self):
    self.assertEqual( pressure("30","in").value("in"), 30.0 )
    self.assertAlmostEqual( pressure("30","in").value("mb"), 1015.92, 2 )
    self.assertAlmostEqual( pressure("30","in").value("hPa"), 1015.92, 2 )

    self.assertEqual( pressure("30","in").string("in"), "30.00 inches" )
    self.assertEqual( pressure("30","in").string("mb"), "1015.9 mb" )
    self.assertEqual( pressure("30","in").string("hPa"), "1015.9 hPa" )
    
    self.assertEqual( pressure("1000","mb").value("mb"), 1000.0 )
    self.assertEqual( pressure("1000","mb").value("hPa"), 1000.0 )
    self.assertAlmostEqual( pressure("1000","mb").value("in"), 29.5299, 4 )
    self.assertAlmostEqual( pressure("1000","hPa").value("in"), 29.5299, 4 )
    
if __name__=='__main__':
  unittest.main()
  



import unittest
import warnings
from datetime import datetime, timedelta

from metar import Metar

# METAR fragments used in tests, below
sta_time = "KEWR 101651Z "
sta_time_mod = "KEWR 101651Z AUTO "
sta_time_wind = "KEWR 101651Z 00000KT "

today = datetime.utcnow()
tomorrow = today + timedelta(days=1)


class MetarTest(unittest.TestCase):
  def raisesParserError(self, code):
    self.assertRaises(Metar.ParserError, Metar.Metar, code )

  def test_issue40_runwayunits(self):
    """Check reported units on runway visual range."""
    report = Metar.Metar(
      "METAR KPIT 091955Z COR 22015G25KT 3/4SM R28L/2600FT TSRA OVC010CB "
      "18/16 A2992 RMK SLP045 T01820159"
    )
    res = report.runway_visual_range()
    self.assertEquals(res, 'on runway 28L, 2600 feet')
    res = report.runway_visual_range('M')
    self.assertTrue(res, 'on runway 28L, 792 meters')

  def test_010_parseType_default(self):
    """Check default value of the report type."""
    self.assertEqual( Metar.Metar("KEWR").type, "METAR" )

  def test_011_parseType_legal(self):
    """Check parsing of the report type."""
    self.assertEqual( Metar.Metar("METAR").type, "METAR" )
    self.assertEqual( Metar.Metar("SPECI").type, "SPECI" )
    self.raisesParserError("TAF" )

  def test_020_parseStation_legal(self):
    """Check parsing of the station code."""
    self.assertEqual( Metar.Metar("KEWR").station_id, "KEWR" )
    self.assertEqual( Metar.Metar("METAR KEWR").station_id, "KEWR" )
    self.assertEqual( Metar.Metar("BIX1").station_id, "BIX1" )
    self.assertEqual( Metar.Metar("K256").station_id, "K256" )

  def test_021_parseStation_illegal(self):
    """Check rejection of illegal station codes."""
    self.raisesParserError( "1ABC" )
    self.raisesParserError( "METAR METAR" )
    self.raisesParserError( "METAR DC" )
    self.raisesParserError( "METAR A" )
    self.raisesParserError( "kewr" )

  def test_030_parseTime_legal(self):
    """Check parsing of the time stamp."""
    report =  Metar.Metar("KEWR 101651Z")
    assert report.decode_completed
    self.assertEqual( report.time.day, 10 )
    self.assertEqual( report.time.hour, 16 )
    self.assertEqual( report.time.minute, 51 )
    if today.day > 10 or (today.hour > 16 and today.day == 10):
        self.assertEqual( report.time.month, today.month )
    if today.month > 1 or today.day > 10:
        self.assertEqual( report.time.year, today.year )

  def test_031_parseTime_specify_year(self):
    """Check that the year can be specified."""
    other_year = 2003

    report =  Metar.Metar("KEWR 101651Z", year=other_year)
    assert report.decode_completed
    self.assertEqual( report.time.year, other_year )

  def test_032_parseTime_specify_month(self):
    """Check that the month can be specified."""
    last_month = ((today.month - 2) % 12) + 1
    last_year = today.year - 1

    report =  Metar.Metar("KEWR 101651Z", month=last_month)
    assert report.decode_completed
    self.assertEqual( report.time.month, last_month )

  def test_033_parseTime_auto_month(self):
    """Check that we assign report to previous month if it can't be in this month."""
    next_day = tomorrow.day
    if next_day > today.day:
        last_month = ((today.month - 2) % 12) + 1
        last_year = today.year - 1

        timestr = "%02d1651Z" % (next_day)
        report =  Metar.Metar("KEWR " + timestr)
        assert report.decode_completed
        self.assertEqual( report.time.day, next_day )
        self.assertEqual( report.time.month, last_month )
        if today.month > 1:
            self.assertEqual( report.time.year, today.year )
        else:
            self.assertEqual( report.time.year, last_year )

  def test_034_parseTime_auto_year(self):
    """Check that year is adjusted correctly if specified month is in the future."""
    next_month = (today.month % 12) + 1
    last_year = today.year - 1

    report =  Metar.Metar("KEWR 101651Z", month=next_month)
    assert report.decode_completed
    self.assertEqual( report.time.month, next_month )
    if next_month > 1:
        self.assertEqual( report.time.year, last_year )
    else:
        self.assertEqual( report.time.year, today.year )

  def test_035_parseTime_suppress_auto_month(self):
    """Check that explicit month suppresses automatic month rollback."""
    next_day = tomorrow.day
    if next_day > today.day:
        last_month = ((today.month - 2) % 12) + 1
        last_year = today.year - 1

        timestr = "%02d1651Z" % (next_day)
        report =  Metar.Metar("KEWR " + timestr, month=1)
        assert report.decode_completed
        self.assertEqual( report.time.day, next_day )
        self.assertEqual( report.time.month, 1 )
        if today.month > 1:
            self.assertEqual( report.time.year, today.year )
        else:
            self.assertEqual( report.time.year, last_year )

  def test_040_parseModifier_default(self):
    """Check default 'modifier' value."""
    self.assertEqual( Metar.Metar("KEWR").mod, "AUTO" )

  def test_041_parseModifier(self):
    """Check parsing of 'modifier' groups."""
    self.assertEqual( Metar.Metar(sta_time+"AUTO").mod, "AUTO" )
    self.assertEqual( Metar.Metar(sta_time+"COR").mod, "COR" )

  def test_042_parseModifier_nonstd(self):
    """Check parsing of nonstandard 'modifier' groups."""

    def report(mod_group):
      """(Macro) Return Metar object from parsing the given modifier group."""
      return Metar.Metar(sta_time+mod_group)

    self.assertEqual( report("RTD").mod, "RTD" )
    self.assertEqual( report("TEST").mod, "TEST" )
    self.assertEqual( report("CCA").mod, "CCA" )
    self.assertEqual( report("CCB").mod, "CCB" )
    self.assertEqual( report("CCC").mod, "CCC" )
    self.assertEqual( report("CCD").mod, "CCD" )
    self.assertEqual( report("CCE").mod, "CCE" )
    self.assertEqual( report("CCF").mod, "CCF" )
    self.assertEqual( report("CCG").mod, "CCG" )
    self.assertEqual( report("CORR").mod, "COR" )
    self.assertEqual( report("FINO").mod, "NO DATA" )
    self.assertEqual( report("NIL").mod, "NO DATA" )

  def test_043_parseModifier_illegal(self):
    """Check rejection of illegal 'modifier' groups."""
    self.raisesParserError( sta_time+"auto" )
    self.raisesParserError( sta_time+"CCH" )
    self.raisesParserError( sta_time+"MAN" )

  def test_140_parseWind(self):
    """Check parsing of wind groups."""
    report = Metar.Metar(sta_time+"09010KT" )
    assert report.decode_completed
    self.assertEqual( report.wind_dir.value(), 90 )
    self.assertEqual( report.wind_speed.value(), 10 )
    self.assertEqual( report.wind_gust, None )
    self.assertEqual( report.wind_dir_from, None )
    self.assertEqual( report.wind_dir_from, None )
    self.assertEqual( report.wind(), "E at 10 knots" )

    report = Metar.Metar(sta_time+"09010MPS" )
    assert report.decode_completed
    self.assertEqual( report.wind_speed.value(), 10 )
    self.assertEqual( report.wind_speed.value("KMH"), 36 )
    self.assertEqual( report.wind(), "E at 19 knots" )
    self.assertEqual( report.wind("MPS"), "E at 10 mps" )
    self.assertEqual( report.wind("KMH"), "E at 36 km/h" )

    report = Metar.Metar(sta_time+"09010KMH" )
    assert report.decode_completed
    self.assertEqual( report.wind_speed.value(), 10 )
    self.assertEqual( report.wind(), "E at 5 knots" )
    self.assertEqual( report.wind('KMH'), "E at 10 km/h" )

    report = Metar.Metar(sta_time+"090010KT" )
    assert report.decode_completed
    self.assertEqual( report.wind_dir.value(), 90 )
    self.assertEqual( report.wind_speed.value(), 10 )

    report = Metar.Metar(sta_time+"000000KT" )
    assert report.decode_completed
    self.assertEqual( report.wind_dir.value(), 0 )
    self.assertEqual( report.wind_speed.value(), 0 )
    self.assertEqual( report.wind(), "calm" )

    report = Metar.Metar(sta_time+"VRB03KT" )
    assert report.decode_completed
    self.assertEqual( report.wind_dir, None )
    self.assertEqual( report.wind_speed.value(), 3 )
    self.assertEqual( report.wind(), "variable at 3 knots" )

    report = Metar.Metar(sta_time+"VRB00KT" )
    assert report.decode_completed
    self.assertEqual( report.wind(), "calm" )

    report = Metar.Metar(sta_time+"VRB03G40KT" )
    assert report.decode_completed
    self.assertEqual( report.wind_dir, None )
    self.assertEqual( report.wind_speed.value(), 3 )
    self.assertEqual( report.wind_gust.value(), 40 )
    self.assertEqual( report.wind_dir_from, None )
    self.assertEqual( report.wind_dir_to, None )
    self.assertEqual( report.wind(), "variable at 3 knots, gusting to 40 knots" )

    report = Metar.Metar(sta_time+"21010G30KT" )
    assert report.decode_completed
    self.assertEqual( report.wind(), "SSW at 10 knots, gusting to 30 knots" )

    report = Metar.Metar(sta_time+"21010KT 180V240" )
    self.assertEqual( report.wind_dir.value(), 210 )
    self.assertEqual( report.wind_speed.value(), 10 )
    self.assertEqual( report.wind_gust, None )
    self.assertEqual( report.wind_dir_from.value(), 180 )
    self.assertEqual( report.wind_dir_to.value(), 240 )
    self.assertEqual( report.wind(), "S to WSW at 10 knots" )

  def test_141_parseWind_nonstd(self):
    """Check parsing of nonstandard wind groups."""

    def report(wind_group):
      """(Macro) Return Metar object from parsing the given wind group."""
      return Metar.Metar(sta_time+wind_group)

    self.assertEqual( report("OOOOOKT").wind_speed.value(), 0 )
    self.assertEqual( report("OOOOOKT").wind(), "calm" )

    self.assertEqual( report("09010K").wind_speed.string(), "10 knots" )
    self.assertEqual( report("09010T").wind_speed.string(), "10 knots" )
    self.assertEqual( report("09010LT").wind_speed.string(), "10 knots" )
    self.assertEqual( report("09010KTS").wind_speed.string(), "10 knots" )
    self.assertEqual( report("09010").wind_speed.string(), "10 mps" )

    self.assertEqual( report("VRBOOK").wind_speed.value(), 0 )
    self.assertEqual( report("VRBOOK").wind(), "calm" )

    self.assertEqual( report("///00KT").wind(), "calm" )
    self.assertEqual( report("/////KT").wind(), "missing" )
    self.assertEqual( report("000//KT").wind(), "missing" )
    self.assertEqual( report("/////").wind(), "missing" )

    self.assertEqual( report("09010G//KT").wind_gust, None )
    self.assertEqual( report("09010GMKT").wind_gust, None )
    self.assertEqual( report("09010GMMKT").wind_gust, None )
    self.assertEqual( report("09010G7KT").wind_gust.value(), 7 )

    self.assertEqual( report("MMM00KT").wind(), "calm" )
    self.assertEqual( report("MMMMMKT").wind(), "missing" )
    self.assertEqual( report("000MMKT").wind(), "missing" )
    self.assertEqual( report("MMMMM").wind(), "missing" )
    self.assertEqual( report("MMMMMGMMKT").wind(), "missing" )
    self.assertEqual( report("MMMMMG01KT").wind(), "missing" )

  def test_issue51_strict(self):
    """Check that setting strict=False prevents a ParserError"""
    with warnings.catch_warnings(record=True) as w:
        report = Metar.Metar(sta_time+"90010KT", strict=False)
    assert len(w) == 1
    assert report.wind_speed is None

  def test_142_parseWind_illegal(self):
    """Check rejection of illegal wind groups."""
    self.raisesParserError( sta_time+"90010KT" )
    self.raisesParserError( sta_time+"9010KT" )
    self.raisesParserError( sta_time+"09010 KT" )
    self.raisesParserError( sta_time+"09010FPS" )
    self.raisesParserError( sta_time+"09010MPH" )
    self.raisesParserError( sta_time+"00///KT" )
    self.raisesParserError( sta_time+"VAR10KT" )
    self.raisesParserError( sta_time+"21010KT 180-240" )
    self.raisesParserError( sta_time+"123UnME" )

  def test_150_parseVisibility(self):
    """Check parsing of visibility groups."""

    def report(vis_group):
      """(Macro) Return Metar object for a report containing the given visibility group."""
      return Metar.Metar(sta_time+"09010KT "+vis_group)

    def report_nowind(vis_group):
      """(Macro) Return Metar object for a report containing the given
      visibility group, without a preceeding wind group.
      """
      return Metar.Metar(sta_time+vis_group)

    self.assertEqual( report("10SM").vis.value(), 10 )
    self.assertEqual( report("10SM").vis_dir, None )
    self.assertEqual( report("10SM").max_vis, None )
    self.assertEqual( report("10SM").max_vis_dir, None )
    self.assertEqual( report("10SM").visibility(), "10 miles" )

    self.assertEqual( report("3/8SM").vis.value(), 0.375 )
    self.assertEqual( report("3/8SM").vis_dir, None )
    self.assertEqual( report("3/8SM").max_vis, None )
    self.assertEqual( report("3/8SM").max_vis_dir, None )
    self.assertEqual( report("3/8SM").visibility(), "3/8 miles" )

    self.assertEqual( report("1 3/4SM").vis.value(), 1.75 )
    self.assertEqual( report("1 3/4SM").vis_dir, None )
    self.assertEqual( report("1 3/4SM").max_vis, None )
    self.assertEqual( report("1 3/4SM").max_vis_dir, None )
    self.assertEqual( report("1 3/4SM").visibility(), "1 3/4 miles" )

    self.assertEqual( report("5000").vis.value(), 5000 )
    self.assertEqual( report("5000").vis_dir, None )
    self.assertEqual( report("5000").visibility(), "5000 meters" )
    self.assertEqual( report("5000M").visibility(), "5000 meters" )

    self.assertEqual( report_nowind("5000").vis.value(), 5000 )
    self.assertEqual( report_nowind("1000W 3000").vis.value(), 1000 )
    self.assertEqual( report_nowind("1000 3000NE").vis.value(), 1000 )

    self.assertEqual( report("CAVOK").vis.value(), 10000 )
    self.assertEqual( report("CAVOK").vis_dir, None )
    self.assertEqual( report("CAVOK").max_vis, None )
    self.assertEqual( report("CAVOK").max_vis_dir, None )
    self.assertEqual( report("CAVOK").visibility(), "10000 meters" )

    self.assertEqual( report("1000W 3000").vis.value(), 1000 )
    self.assertEqual( report("1000W 3000").vis_dir.value(), 270 )
    self.assertEqual( report("1000W 3000").max_vis.value(), 3000 )
    self.assertEqual( report("1000W 3000").max_vis_dir, None )
    self.assertEqual( report("1000W 3000").visibility(), "1000 meters to W; 3000 meters" )

    self.assertEqual( report("1000 3000NE").vis.value(), 1000 )
    self.assertEqual( report("1000 3000NE").vis_dir, None )
    self.assertEqual( report("1000 3000NE").max_vis.value(), 3000 )
    self.assertEqual( report("1000 3000NE").max_vis_dir.value(), 45 )
    self.assertEqual( report("1000 3000NE").visibility(), "1000 meters; 3000 meters to NE" )

    self.assertEqual( report("5KM").vis.value(), 5 )
    self.assertEqual( report("5KM").vis_dir, None )
    self.assertEqual( report("5KM").visibility(), "5.0 km" )

    self.assertEqual( report("5000E").vis.value(), 5000 )
    self.assertEqual( report("5000E").visibility(), "5000 meters to E" )

    self.assertEqual( report("7000NDV").vis.value(), 7000 )
    self.assertEqual( report("7000NDV").vis_dir, None )
    self.assertEqual( report("7000NDV").visibility(), "7000 meters" )

    self.assertEqual( report("M1000").vis.value(), 1000 )
    self.assertEqual( report("M1000").visibility(), "less than 1000 meters" )

    self.assertEqual( report("P6000").vis.value(), 6000 )
    self.assertEqual( report("P6000").visibility(), "greater than 6000 meters" )

  def test_151_parseVisibility_direction(self):
    """Check parsing of compass headings visibility groups."""

    def report(vis_group):
      """(Macro) Return Metar object for a report containing the given visibility group."""
      return Metar.Metar(sta_time+"09010KT "+vis_group)

    self.assertEqual( report("5000N").vis_dir.compass(), "N" )
    self.assertEqual( report("5000N").vis_dir.value(), 0 )
    self.assertEqual( report("5000NE").vis_dir.compass(), "NE" )
    self.assertEqual( report("5000NE").vis_dir.value(), 45 )
    self.assertEqual( report("5000E").vis_dir.compass(), "E" )
    self.assertEqual( report("5000E").vis_dir.value(), 90 )
    self.assertEqual( report("5000SE").vis_dir.compass(), "SE" )
    self.assertEqual( report("5000SE").vis_dir.value(), 135 )
    self.assertEqual( report("5000S").vis_dir.compass(), "S" )
    self.assertEqual( report("5000S").vis_dir.value(), 180 )
    self.assertEqual( report("5000SW").vis_dir.compass(), "SW" )
    self.assertEqual( report("5000SW").vis_dir.value(), 225 )
    self.assertEqual( report("5000W").vis_dir.compass(), "W" )
    self.assertEqual( report("5000W").vis_dir.value(), 270 )
    self.assertEqual( report("5000NW").vis_dir.compass(), "NW" )
    self.assertEqual( report("5000NW").vis_dir.value(), 315 )

  def test_152_parseVisibility_with_following_temperature(self):
    """Check parsing of visibility groups followed immediately by a temperature group."""

    def report(vis_group):
      """(Macro) Return Metar object for a report containing the given visibility group."""
      return Metar.Metar(sta_time+"09010KT "+vis_group)

    self.assertEqual( report("CAVOK 02/01").vis.value(), 10000 )
    self.assertEqual( report("CAVOK 02/01").vis_dir, None )
    self.assertEqual( report("CAVOK 02/01").max_vis, None )
    self.assertEqual( report("CAVOK 02/01").temp.value(), 2.0 )
    self.assertEqual( report("CAVOK 02/01").dewpt.value(), 1.0 )

    self.assertEqual( report("5000 02/01").vis.value(), 5000 )
    self.assertEqual( report("5000 02/01").vis_dir, None )
    self.assertEqual( report("5000 02/01").max_vis, None )
    self.assertEqual( report("5000 02/01").temp.value(), 2.0 )
    self.assertEqual( report("5000 02/01").dewpt.value(), 1.0 )

  def test_290_ranway_state(self):
    """Check parsing of runway state groups."""

    def report(runway_state):
      """(Macro) Return Metar object for a report containing the given runway state group"""
      sample_metar = 'EGNX 191250Z VRB03KT 9999 -RASN FEW008 SCT024 BKN046 M01/M03 Q0989 '
      return Metar.Metar(sample_metar+' '+runway_state)

    self.assertEqual( report('09690692 27550591').temp.value(), -1.0 )
    self.assertEqual( report('09690692 27550591').remarks(), "" )

    self.assertEqual( report('09SNOCLO').remarks(), "" )
    self.assertEqual( report('09CLRD//').remarks(), "" )

  def test_300_parseTrend(self):
    """Check parsing of trend forecasts."""

    def report(trend_group, remarks=""):
      """(Macro)
      Return Metar object for a report containing the given trend
      forecast and remarks.
      """
      sample_metar = sta_time+"09010KT 10SM -SN OVC020 23/05 Q1001"
      return Metar.Metar(sample_metar+' '+trend_group+' '+remarks)

    self.assertEqual( report('TEMPO FM0306 BKN030CU').trend(), 'TEMPO FM0306 BKN030CU' )
    self.assertEqual( report('TEMPO FM0306 BKN030CU').temp.value(), 23.0 )
    self.assertEqual( report('TEMPO FM0306 BKN030CU').remarks(), "" )

    self.assertEqual( report('BECMG 0306 VRB06KT').trend(), 'BECMG 0306 VRB06KT' )
    self.assertEqual( report('FCST AT0327 +FC').trend(), 'FCST AT0327 +FC' )

    self.assertEqual( report('TEMPO 0306 1/2SM').trend(), 'TEMPO 0306 1/2SM' )
    self.assertEqual( report('TEMPO FM0306 TL0345 01030G50KT').trend(),
                      'TEMPO FM0306 TL0345 01030G50KT')

    self.assertEqual( report('TEMPO 0306 RMK 402500072').trend(), 'TEMPO 0306' )
    self.assertEqual( report('TEMPO 0306 RMK 402500072').max_temp_24hr.value(), 25.0 )

  def test_snowdepth(self):
    """Check parsing of 4/ group snowdepth"""
    sample_metar = ("KDOV 040558Z 23004KT 1 1/2SM R01/2800FT -SN BR "
                    "OVC006 M01/M01 A3015 RMK AO2A SLP213 P0000 4/001 "
                    "60010 T10071007 10017 "
                    "21009 55016 VISNO RWY19 CHINO RWY19 $")
    m = Metar.Metar(sample_metar)
    self.assertEquals(m.snowdepth.value(), 1)

  def test_310_parse_sky_conditions(self):
    """Check parsing of sky conditions"""

    def report(sky_conditions):
      """(Macro) Return Metar object for a report containing the given sky conditions"""
      sample_metar = "{} 14005KT 6000 {} M05/M10 Q1018".format(sta_time, sky_conditions)
      return Metar.Metar(sample_metar)

    self.assertEqual( report('SCT030').sky_conditions(), 'scattered clouds at 3000 feet' )
    self.assertEqual( report('BKN001').sky_conditions(), 'broken clouds at 100 feet' )
    self.assertEqual( report('OVC008').sky_conditions(), 'overcast at 800 feet' )
    self.assertEqual(
      report('OVC010CB').sky_conditions(), 'overcast cumulonimbus at 1000 feet'
    )
    self.assertEqual( report('SCT020TCU').sky_conditions(), 'scattered towering cumulus at 2000 feet' )
    self.assertEqual( report('BKN015CB').sky_conditions(), 'broken cumulonimbus at 1500 feet' )
    self.assertEqual( report('FEW030').sky_conditions(), 'a few clouds at 3000 feet' )
    self.assertEqual( report('VV001').sky_conditions(), 'indefinite ceiling, vertical visibility to 100 feet' )
    self.assertEqual( report('SKC').sky_conditions(), 'clear' )
    self.assertEqual( report('CLR').sky_conditions(), 'clear' )
    self.assertEqual( report('NSC').sky_conditions(), 'clear' )

  def test_not_strict_mode(self):
    # This example metar has an extraneous 'M' in it, but the rest is fine
    # Let's make sure that we can activate a non-strict mode, and flag that there
    # are unparsed portions
    code = 'K9L2 100958Z AUTO 33006KT 10SM CLR M A3007 RMK AO2 SLPNO FZRANO $'
    self.raisesParserError(code)

    with warnings.catch_warnings(record=True) as w:
        report = Metar.Metar(code, strict=False)
    assert len(w) == 1

    assert not report.decode_completed
    self.assertEqual( report.cycle, 10 )
    self.assertEqual( report.mod, 'AUTO' )
    self.assertEqual( report.recent, [] )
    self.assertEqual( report.station_id, 'K9L2' )
    self.assertEqual( report.vis.value(), 10 )
    self.assertEqual( report.sky_conditions(), 'clear' )


if __name__=='__main__':
  unittest.main()



import unittest
from metar.Datatypes import distance, UnitsError

class DistanceTest(unittest.TestCase):
  
  def testDefaults(self):
    self.assertEqual( distance("10").value(), 10.0 )
    self.assertEqual( distance("1000").value("M"), 1000.0 )
    self.assertEqual( distance("1500").string(), "1500 meters" )
    self.assertEqual( distance("1500",None).string(), "1500 meters" )
    self.assertEqual( distance("5","SM").string(), "5 miles" )
  
  def testInputs(self):
    self.assertEqual( distance("10").value(), 10.0 )
    self.assertEqual( distance(10).value(), 10.0 )
    self.assertEqual( distance(10.0).value(), 10.0 )
    self.assertEqual( distance(10.0, None).value(), 10.0 )
    self.assertEqual( distance("1/2").value(), 0.5 )
    self.assertEqual( distance("1 1/2").value(), 1.5 )
    self.assertEqual( distance("11/2").value(), 1.5 )
    self.assertEqual( distance("10", gtlt=">").value(), 10.0 )
    self.assertEqual( distance("10", None, "<").value(), 10.0 )
  
  def testErrorChecking(self):
    self.assertRaises( ValueError, distance, "10SM" )
    self.assertRaises( ValueError, distance, "M1/2SM" )
    self.assertRaises( ValueError, distance, "1000", "M", "=" )
    self.assertRaises( ValueError, distance, "1000", "M", "gt" )
    self.assertRaises( UnitsError, distance, "10", "NM" )
    self.assertRaises( UnitsError, distance("1000").value, "furlongs" )
    self.assertRaises( UnitsError, distance("500").string, "yards" )
  
  def testConversions(self):
    self.assertEqual( distance("5","SM").value("SM"), 5.0 )
    self.assertEqual( distance("5","SM").value("MI"), 5.0 )
    self.assertAlmostEqual( distance("5","SM").value("M"), 8046.7, 1 )
    self.assertAlmostEqual( distance("5","SM").value("KM"), 8.05, 2 )
    self.assertAlmostEqual( distance("5","SM").value("FT"), 26400.0, 1 )
 
    self.assertEqual( distance("5000","M").value("M"), 5000.0 )
    self.assertEqual( distance("5000","M").value("KM"), 5.0 )
    self.assertAlmostEqual( distance("5000","M").value("SM"), 3.1, 1 )
    self.assertAlmostEqual( distance("5000","M").value("MI"), 3.1, 1 )
    self.assertAlmostEqual( distance("5000","M").value("FT"), 16404.0, 0 )
 
    self.assertEqual( distance("5","KM").value("KM"), 5.0 )
    self.assertEqual( distance("5","KM").value("M"), 5000.0 )
    self.assertAlmostEqual( distance("5","KM").value("SM"), 3.1, 1 )
    self.assertAlmostEqual( distance("5","KM").value("FT"), 16404.0, 0 )
 
    self.assertEqual( distance("5280","FT").value("FT"), 5280.0 )
    self.assertAlmostEqual( distance("5280","FT").value("SM"), 1.0, 5 )
    self.assertAlmostEqual( distance("5280","FT").value("MI"), 1.0, 5 )
    self.assertAlmostEqual( distance("5280","FT").value("KM"), 1.609, 3 )
    self.assertAlmostEqual( distance("5280","FT").value("M"), 1609, 0 )
    
    self.assertAlmostEqual( distance("1 1/2","SM").value("FT"), 7920.0, 2 )
    self.assertAlmostEqual( distance("1/4","SM").value("FT"), 1320.0, 2 )

    self.assertAlmostEqual( distance("10.5","IN").value("M"), 0.27, 2 )
    self.assertAlmostEqual( distance("0.066","KM").value("IN"), 2598.43, 2 )

    self.assertEqual( distance("1 1/2","SM").string("SM"), "1 1/2 miles" )
    self.assertEqual( distance("3/16","SM").string("SM"), "3/16 miles" )
    self.assertEqual( distance("1/4","SM").string("FT"), "1320 feet" )
    self.assertEqual( distance("1/4","SM","<").string("SM"), "less than 1/4 miles" )
    self.assertEqual( distance("5280","FT").string("KM"), "1.6 km" )
    self.assertEqual( distance("10000","M",">").string("M"), "greater than 10000 meters" )


if __name__=='__main__':
  unittest.main()
  



"""Setup script for the metar package.

    Usage: python setup.py install
"""
from setuptools import setup
from metar import __version__

DESCRIPTION = "Metar - a package to parse METAR-coded weather reports"

LONG_DESCRIPTION = """
Metar is a python package for interpreting METAR and SPECI weather reports.

METAR is an international format for reporting weather observations.
The standard specification for the METAR and SPECI codes is given
in the WMO Manual on Codes, vol I.1, Part A (WMO-306 I.i.A).  US
conventions for METAR/SPECI reports are described in chapter 12 of
the Federal Meteorological Handbook No.1. (FMH-1 1995), issued by
NOAA.  See http://www.ncdc.noaa.gov/oa/wdc/metar/

This module extracts the data recorded in the main-body groups of
reports that follow the WMO spec or the US conventions, except for
the runway state and trend groups, which are parsed but ignored.
The most useful remark groups defined in the US spec are parsed,
as well, such as the cumulative precipitation, min/max temperature,
peak wind and sea-level pressure groups.  No other regional conventions
are formally supported, but a large number of variant formats found
in international reports are accepted."""

setup(
    name="metar",
    version=__version__,
    author="Tom Pollard",
    author_email="pollard@alum.mit.edu",
    url="https://github.com/python-metar/python-metar",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license="MIT",
    packages=["metar"],
    package_data={'metar': ['nsd_cccc.txt', ]},
    platforms="Python 2.5 and later.",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Intended Audience :: Science/Research",
        # "Topic :: Formats and Protocols :: Data Formats",
        # "Topic :: Scientific/Engineering :: Earth Sciences",
        # "Topic :: Software Development :: Libraries :: Python Modules"
        ]
    )



import unittest
from metar.Datatypes import speed, UnitsError

class SpeedTest(unittest.TestCase):
  
  def testDefaults(self):
    self.assertEqual( speed("10").value(), 10.0 )
    self.assertEqual( speed("5").string(), "5 mps" )
    self.assertEqual( speed("10", "KT").value(), 10.0 )
    self.assertEqual( speed("5", "KT").string(), "5 knots" )
    self.assertEqual( speed("5", "KMH").string(), "5 km/h" )
    self.assertEqual( speed("5", "MPH").string(), "5 mph" )
    self.assertEqual( speed("5",None).string(), "5 mps" )
  
  def testInputs(self):
    self.assertEqual( speed("10").value(), 10.0 )
    self.assertEqual( speed(10).value(), 10.0 )
    self.assertEqual( speed(10.0).value(), 10.0 )
    self.assertEqual( speed(10.0, None).value(), 10.0 )
    self.assertEqual( speed("10", gtlt=">").value(), 10.0 )
    self.assertEqual( speed("10", None, "<").value(), 10.0 )
  
  def testErrorChecking(self):
    self.assertRaises( ValueError, speed, "10KT" )
    self.assertRaises( ValueError, speed, "10", "MPS", "=" )
    self.assertRaises( ValueError, speed, "60", "KT", "gt" )
    self.assertRaises( UnitsError, speed, "10", "NM" )
    self.assertRaises( UnitsError, speed("10").value, "furlongs per fortnight" )
    self.assertRaises( UnitsError, speed("5").string, "fps" )
  
  def testConversions(self):
    self.assertEqual( speed("10", "MPS").value("MPS"), 10.0 )
    self.assertEqual( speed("10", "MPS").value("KMH"), 36.0 )
    self.assertAlmostEqual( speed("10", "MPS").value("MPH"), 22.4, 1 )
    self.assertAlmostEqual( speed("10", "MPS").value("KT"), 19.4, 1 )
    
    self.assertEqual( speed("10", "KT").value("KT"), 10.0 )
    self.assertAlmostEqual( speed("10", "KT").value("MPH"), 11.5, 1 )
    self.assertAlmostEqual( speed("10", "KT").value("MPS"), 5.1, 1 )
    self.assertAlmostEqual( speed("10", "KT").value("KMH"), 18.5, 1 )
    
    self.assertEqual( speed("10", "MPH").value("MPH"), 10.0 )
    self.assertAlmostEqual( speed("10", "MPH").value("KT"), 8.7, 1 )
    self.assertAlmostEqual( speed("10", "MPH").value("MPS"), 4.5, 1 )
    self.assertAlmostEqual( speed("10", "MPH").value("KMH"), 16.1, 1 )
    
    self.assertEqual( speed("10", "KMH").value("KMH"), 10.0 )
    self.assertAlmostEqual( speed("10", "KMH").value("KT"), 5.4, 1 )
    self.assertAlmostEqual( speed("10", "KMH").value("MPS"), 2.8, 1 )
    self.assertAlmostEqual( speed("10", "KMH").value("MPH"), 6.2, 1 )
    
if __name__=='__main__':
  unittest.main()
  
